package de.fzj.unicore.bes.security;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.addressing.AddressingInHandler;
import org.codehaus.xfire.handler.AbstractHandler;
import org.codehaus.xfire.handler.Phase;
import org.jdom.Element;
import org.jdom.Namespace;

import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.xmlbeans.BaseFault;
import eu.unicore.security.SecurityTokens;
import eu.unicore.security.xfireutil.AuthInHandler;

/**
 * @deprecated
 */
public class UsernameInHandler extends AbstractHandler {

	private static final Logger logger = LogUtil.getLogger(LogUtil.SECURITY,UsernameInHandler.class);

	protected ISecurityProperties sec;
	
	protected FileBasedUsernameValidator fileValidator;

	private boolean performAuthCheck=true;

	private final Kernel kernel;
	
	/**
	 * creates a new handler for validating username/password  
	 */
	public UsernameInHandler(Kernel kernel) {
		super();
		this.kernel=kernel;
		setPhase(Phase.POLICY);
		after(AddressingInHandler.class.getName());
		after(AuthInHandler.class.getName());	
	}

	public void invoke(MessageContext ctx) throws BaseFault {

		logger.debug("+++++++username/password in handler invoked++++++++");

		Element h = ctx.getInMessage().getOrCreateHeader();

		Map<String, String> map = extractUsernameAndPassword(h);

		String uname = map.get(SecurityConstant.USERNAME);
		String password = map.get(SecurityConstant.PASSWORD);

		//store for later use
		SecurityTokens tokens=(SecurityTokens)ctx.getProperty(SecurityTokens.KEY);
		if(tokens!=null){
			tokens.getContext().put(SecurityConstant.WSS_USERNAME_SECURITY_TOKEN+".username", uname);
			tokens.getContext().put(SecurityConstant.WSS_USERNAME_SECURITY_TOKEN+".password", password);
		}

		if(logger.isDebugEnabled()){
			logger.debug("username: "+uname);
		}

		if(performAuthCheck){
			if(uname.equals("") || password.equals("")){
				logger.fatal("Empty username/password");
				throw new BaseFault("Access denied: Empty username/password");
			}
			try {
				fileValidator = FileBasedUsernameValidator.getInstance(kernel.getProperty(FileBasedUsernameValidator.USERNAME_MAP_FILE));
			} catch (Exception e) {
				throw new BaseFault("Username mapping error. Contact service administrator.");
			}
			if(!fileValidator.validate(uname, password)){
				logger.fatal("Wrong username/password");
				throw new BaseFault("username/password doesn't match");
			}
		}
	}

	/**
	 * extract security things from the "wss:Security" header
	 * 
	 * @param header
	 * @return map
	 */
	protected Map<String, String> extractUsernameAndPassword(Element header) {
		Map<String, String> map = new HashMap<String, String>();

		Element sec = header.getChild(SecurityConstant.WSS_SECURITY, Namespace.getNamespace(SecurityConstant.WSS));
		Element utoken = null;
		Element uname = null;
		Element pwd = null;
		try{
			utoken = sec.getChild(SecurityConstant.WSS_USERNAME_SECURITY_TOKEN, Namespace.getNamespace(SecurityConstant.WSS));
			uname = utoken.getChild(SecurityConstant.USERNAME, Namespace.getNamespace(SecurityConstant.WSS));
			pwd = utoken.getChild(SecurityConstant.PASSWORD, Namespace.getNamespace(SecurityConstant.WSS));
			map.put(SecurityConstant.USERNAME, uname.getText());
			map.put(SecurityConstant.PASSWORD, pwd.getText());
		}
		catch(Exception e){
			LogUtil.logException("Error in username/password header element",e);
		}
		return map;
	}

	public boolean isPerformAuthCheck() {
		return performAuthCheck;
	}

	/**
	 * choose whether the handler should perform a auth check during invocation
	 * @param performAuthCheck - if <code>true</code>, the handler will validate 
	 *        username/password and throw a fault upon failure
	 */
	public void setPerformAuthCheck(boolean performAuthCheck) {
		this.performAuthCheck = performAuthCheck;
	}






}
