package de.fzj.unicore.bes.functional;

import java.util.Properties;

import junit.framework.TestCase;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.bes.client.ActivityClient;
import de.fzj.unicore.bes.client.FactoryClient;
import de.fzj.unicore.bes.security.UsernameOutHandler;
import de.fzj.unicore.bes.util.BESOnStartup;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.security.UASSecurityProperties;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;

public class BaseActivityTest extends TestCase{
	
	protected boolean useProxyCert = false;
	protected boolean securityFlag = true; 
	protected static ISecurityProperties sp = null;
	protected String protocol = "http";
	protected String serverUrl = null;
	protected static final String hostAddress = "://localhost:7777/services/";
	
	
	
	protected void configureSecurity() throws Exception{
		if(getSecurityFlag()){
			 protocol = "https";
			 if(useProxyCert)sp = makeSecurityPropertiesWithProxy();
			 else sp = makeSecurityProperties();
		}	
		serverUrl = protocol+hostAddress;
	}
	
	protected FactoryClient getDefaultFactory() throws Exception{
		configureSecurity();

		String factoryUrl = serverUrl+"BESFactory";
		EndpointReferenceType eprt = EndpointReferenceType.Factory.newInstance();
		eprt.addNewAddress().setStringValue(factoryUrl+"?res="+BESOnStartup.INSTANCE_ID);

		FactoryClient client = null;
		try {
			client = new FactoryClient(factoryUrl, eprt, sp);
			return client;
		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}
		
		
		return null;
	}
	
	protected EndpointReferenceType getDefaultFactoryEpr(){
		EndpointReferenceType eprt = EndpointReferenceType.Factory.newInstance();
		eprt.addNewAddress().setStringValue(serverUrl+"BESFactory?res="+BESOnStartup.INSTANCE_ID);
		WSServerUtilities.addUGSRefparamToEpr(eprt, BESOnStartup.INSTANCE_ID);
		return eprt;
	}
		
	protected ActivityClient getActivityClient(EndpointReferenceType epr) throws Exception {
		configureSecurity();
		String activityUrl = serverUrl+"BESActivity";
		
		try {
			ActivityClient ac = new ActivityClient(activityUrl, epr, sp);
			return ac;
		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}
		
		return null;
	}
	
	
	


	//programatically setup security
	protected ISecurityProperties makeSecurityProperties() throws Exception {
		if(sp==null){
			String certs="src/main/certs";
			Properties p = new Properties();
			p.setProperty(ISecurityProperties.WSRF_SSL,"true");
//			p.setProperty(ISecurityProperties.WSRF_WSS,"false");
			p.setProperty(ISecurityProperties.WSRF_SSL_CLIENTAUTH,"true");
			p.setProperty(ISecurityProperties.WSRF_SSL_KEYSTORE,certs+"/user-keystore.jks");
			p.setProperty(ISecurityProperties.WSRF_SSL_KEYTYPE,"JKS");
			p.setProperty(ISecurityProperties.WSRF_SSL_KEYPASS,"the!user");
			p.setProperty(ISecurityProperties.WSRF_SSL_TRUSTPASS,"the!user");
			p.setProperty(ISecurityProperties.WSRF_SSL_TRUSTSTORE,certs+"/user-keystore.jks");
			p.setProperty(ISecurityProperties.WSRF_SSL_TRUSTTYPE,"JKS");
			
			p.setProperty(ISecurityProperties.UAS_OUTHANDLER_NAME,UsernameOutHandler.class.getName());
			UASSecurityProperties secp=new UASSecurityProperties(p);

			secp.setSignMessage(true);
			secp.getETDSettings().setExtendTrustDelegation(true);
			
//			String dn = "C=DE,ST=none,L=none,O=Unicore,OU=none,CN=NJS Test Certificate,1.2.840.113549.1.9.1=#161c756e69636f72652d737570706f7274406c697374732e73662e6e6574";
//			secpp.setReceiver(new javax.security.auth.x500.X500Principal(dn));
		
			
			sp=secp;
		}
		return sp;
	}


	//programatically setup proxy credentials
	protected ISecurityProperties makeSecurityPropertiesWithProxy() throws Exception{
		if(sp==null){
			Properties p = new Properties();
			String certs="src/main/certs";
			p.setProperty(ISecurityProperties.WSRF_SSL,"true");
			p.setProperty(ISecurityProperties.WSRF_SSL_CLIENTAUTH,"true");
			p.setProperty(ISecurityProperties.WSRF_SSL_KEYSTORE,"/tmp/proxy/user-proxy_aa.p12");
			p.setProperty(ISecurityProperties.WSRF_SSL_KEYTYPE,"pkcs12");
			p.setProperty(ISecurityProperties.WSRF_SSL_KEYPASS,"the!user");
			p.setProperty(ISecurityProperties.WSRF_SSL_TRUSTPASS,"the!user");
			p.setProperty(ISecurityProperties.WSRF_SSL_TRUSTSTORE,certs+"/user-keystore.jks");
			p.setProperty(ISecurityProperties.WSRF_SSL_TRUSTTYPE,"JKS");
			UASSecurityProperties secp=new UASSecurityProperties(p);			
			secp.setSignMessage(false);
			secp.getETDSettings().setExtendTrustDelegation(true);
			
//			String dn = "C=DE,ST=none,L=none,O=Unicore,OU=none,CN=NJS Test Certificate,1.2.840.113549.1.9.1=#161c756e69636f72652d737570706f7274406c697374732e73662e6e6574";
//			secp.setReceiver(new javax.security.auth.x500.X500Principal(dn));
			
			sp=secp;
		}
		return sp;
	}
	
	
	protected void setSecurityFlag(boolean b){
		securityFlag = b; 
	}
	
	protected boolean getSecurityFlag(){
		return securityFlag;
	}
	
	
	
	
	
}
