package de.fzj.unicore.bes.security;

import static org.junit.Assert.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import javax.xml.namespace.QName;

import org.codehaus.xfire.service.Service;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.oasisOpen.docs.wsrf.rp2.GetResourcePropertyDocument;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.security.UASSecurityProperties;
import de.fzj.unicore.wsrflite.xfire.WSRFClientFactory;
import de.fzj.unicore.wsrflite.xfire.XFireKernel;
import de.fzj.unicore.wsrflite.xfire.XFireServiceFactory;
import eu.unicore.security.SecurityTokens;

/**
 * TODO remove
 */
public class TestUsernameTokenHandlers {

	private static final String TESTUSERNAME = "user1"; 
	private static final String TESTPASSWORD = "user1";
	private static final String TESTSERVICENAME = "UsernameTokenHandlerService";
	
	static Kernel kernel;
	
	@BeforeClass
	public static void startServer()throws Exception{
		UAS uas=new UAS("src/test/resources/secure/uas.config.simple");
		uas.startSynchronous();
		kernel=uas.getKernel();
		kernel.setProperty(FileBasedUsernameValidator.USERNAME_MAP_FILE, "src/test/resources/secure/users.xml");
		XFireServiceFactory.createAndDeployService(kernel,TESTSERVICENAME,IExample.class, ExampleService.class, false);
		Service s=XFireKernel.get(kernel).getService(TESTSERVICENAME);
		s.addInHandler(new UsernameInHandler(kernel));
	}

	private static String secPropsOut=
		ISecurityProperties.WSRF_SSL + "=true"+"\n"+
		ISecurityProperties.WSRF_SSL_CLIENTAUTH + "=true" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYPASS + "=the!user"+"\n"+
		ISecurityProperties.WSRF_SSL_KEYTYPE + "=jks"+"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTPASS + "=the!user\n";


	@Test
	public void testOutHandler() throws Exception{
		// establish security
		ISecurityProperties sec=new UASSecurityProperties(kernel.getProperties(), 
				new ByteArrayInputStream(secPropsOut.getBytes()));

		sec.setSignMessage(false);
		sec.getExtraSettings().put(SecurityConstant.ATTR_USERNAME, TESTUSERNAME);
		sec.getExtraSettings().put(SecurityConstant.ATTR_PASSWORD, TESTPASSWORD);
		UsernameOutHandler p=new UsernameOutHandler();
		p.configure(sec);
		assertTrue(p.getUserName().equals(TESTUSERNAME));
		assertTrue(p.getPassword().equals(TESTPASSWORD));
		Element ph=p.getUsernameTokenHeader();
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		new XMLOutputter(Format.getPrettyFormat()).output(ph,bos);
		String bs = bos.toString();
		assertTrue(bs.contains(SecurityConstant.WSS_USERNAME_SECURITY_TOKEN));
		assertTrue(bs.contains(TESTUSERNAME));
		bos.close();
	}

	@Test
	public void testInHandler() throws Exception{
		ISecurityProperties sec=new UASSecurityProperties(kernel.getProperties(),
				new ByteArrayInputStream(secPropsOut.getBytes()));
		sec.setProperty(ISecurityProperties.UAS_OUTHANDLER_NAME, UsernameOutHandler.class.getName());
		sec.setSignMessage(false);
		sec.getExtraSettings().put(SecurityConstant.ATTR_USERNAME, TESTUSERNAME);
		sec.getExtraSettings().put(SecurityConstant.ATTR_PASSWORD, TESTPASSWORD);
		sec.setProperty(SecurityConstant.ATTR_ACTIVATEUSERNAMEPROFILE, "true");


		IExample example = new WSRFClientFactory(sec).createPlainWSProxy(IExample.class,kernel.getBaseURL()+"/"+TESTSERVICENAME);
		GetResourcePropertyDocument in=GetResourcePropertyDocument.Factory.newInstance();
		in.setGetResourceProperty(new QName("Hello","World"));
		example.getTime(in);

		SecurityTokens st=ExampleService.getLastCallSecurityTokens();
		assertNotNull(st);
		String userName= (String)st.getContext().get(SecurityConstant.WSS_USERNAME_SECURITY_TOKEN+".username");
		String password=(String) st.getContext().get(SecurityConstant.WSS_USERNAME_SECURITY_TOKEN+".password");
		assertEquals(userName, TESTUSERNAME);
		assertEquals(password, TESTPASSWORD);
	}
	
	@AfterClass
	public static void stopServer(){
		kernel.shutdown();
	}
	
}
