package de.fzj.unicore.uas.impl.dsms;

import java.io.FileNotFoundException;
import java.util.EmptyStackException;
import java.util.Stack;
import java.util.concurrent.Callable;

import org.apache.log4j.Logger;

import pl.edu.icm.dsms.catalogue.CatalogueClient;
import pl.edu.icm.dsms.catalogue.CatalogueUtils;
import pl.edu.icm.dsms.catalogue.model.Lda;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Kernel;
import eu.unicore.security.Client;
import eu.unicore.security.util.Log;

class CheckFileOnSmsCallable implements Callable<Lda>
{
	private static final Logger logger = Log.getLogger(LogUtil.SERVICES, CheckFileOnSmsCallable.class);

	private CatalogueClient cat;
	private String path;
	private Stack<Lda> ldas;
	private String physical;
	private Client client;
	private String dSmsId;
	private Kernel kernel;
	
	public CheckFileOnSmsCallable(Stack<Lda> ldas, 
			CatalogueClient cat, 
			String path, String dSmsId, String xlogin, 
			Client client, Kernel kernel)
	{
		this.cat = cat;
		this.path = path;
		this.ldas = ldas;
		this.physical = CatalogueUtils.getPhysicalName(path, dSmsId, xlogin);
		this.client = client;
		this.dSmsId = dSmsId;
		this.kernel = kernel;
	}
	
	@Override
	public Lda call() throws Exception
	{
		while(true)
		{
			Lda lda;
			try
			{
				lda = ldas.pop();
			}
			catch(EmptyStackException e) { return null; }
			logger.debug("Looking for " + physical + " on " + lda.getSmsAddress());
			StorageClient storage = Utils.getStorageClient(lda, client, kernel);
			try
			{
				storage.listProperties(physical);
			}
			catch(FileNotFoundException e) { continue; }
			
			return cat.add(path, dSmsId, lda.getSms());
		}
	}
}
