package de.fzj.unicore.uas.impl.dsms;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.impl.UASWSResourceImpl;
import de.fzj.unicore.uas.impl.sms.StorageManagementHomeImpl;
import de.fzj.unicore.uas.impl.sms.StorageManagementHomeImpl.StorageTypes;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.KernelInjectable;
import de.fzj.unicore.wsrflite.exceptions.ResourceNotCreatedException;
import de.fzj.unicore.wsrflite.exceptions.ResourceUnknownException;
import de.fzj.unicore.wsrflite.xmlbeans.impl.WSResourceImpl;

public class CreateDSMSOnStartup implements Runnable, KernelInjectable
{
	private static final Logger logger = LogUtil.getLogger(LogUtil.UNICORE, CreateDSMSOnStartup.class);

	public static final String DEFAULT_DSMS_NAME = "default_dsms";

	private Kernel kernel;
	
	public CreateDSMSOnStartup()
	{
	}

	public void setKernel(Kernel kernel){
		this.kernel=kernel;
	}
	
	public void run()
	{
		logger.debug("Creating " + DEFAULT_DSMS_NAME);
		try
		{ 
			Home smsHome = kernel.getHome(UAS.SMS);
			if(smsHome == null)
			{
				logger.info("No dSMS configured for this site.");
				return;
			}

			try
			{
					if(smsHome.get(DEFAULT_DSMS_NAME) != null)
					{
						logger.info(DEFAULT_DSMS_NAME + " already exists");
						return;
					}
			}
			catch(ResourceUnknownException e){}
			doCreateSMS(smsHome);
		}
		catch(Exception e)
		{
			logger.error("Could not add " + DEFAULT_DSMS_NAME, e);
		}
	}
	
	private void doCreateSMS(Home smsHome) throws ResourceNotCreatedException
	{
		String defaultDSmsName = DEFAULT_DSMS_NAME;
		Map<String,Object> imap = new HashMap<String,Object>();
		imap.put(WSResourceImpl.INIT_UNIQUE_ID, defaultDSmsName);
		
		Calendar c = new GregorianCalendar();
		c.add(Calendar.MONTH, 72);
		imap.put(WSResourceImpl.INIT_INITIAL_TERMINATION_TIME, c);
		imap.put(UASWSResourceImpl.INITPARAM_PUBLISH_TO_REGISTRY, true);
		imap.put(StorageManagementHomeImpl.INIT_STORAGETYPE, StorageTypes.DSMS);
		smsHome.createWSRFServiceInstance(imap);
		logger.info("Added dSMS resource " + defaultDSmsName + ".");
	}
}