/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licensing information.
 *
 * Author: T. Rękawek <newton@mat.uni.torun.pl>
 */

package de.fzj.unicore.uas.impl.dsms;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.FileSystemDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.FileSystemTypeEnumeration;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.RangeValueType;

import de.fzj.unicore.wsrflite.impl.ResourceImpl;
import de.fzj.unicore.wsrflite.xmlbeans.ResourceProperty;

/**
 * DSMS resource properties.
 * @author newton
 *
 */
public class DSMSFileSystemRP extends ResourceProperty<Object>
{
	private static final long serialVersionUID=1L;
	   
	private final FileSystemDocument[] xdoc;
	public static final String DESCRIPTION = "DSMS";
	
	public DSMSFileSystemRP(ResourceImpl parent, String name)
	{
		super(parent);
		FileSystemDocument fs = FileSystemDocument.Factory.newInstance();
		fs.addNewFileSystem().setName(name);
		fs.getFileSystem().setFileSystemType(FileSystemTypeEnumeration.NORMAL);
		xdoc = new FileSystemDocument[]{fs};
	}
	
	@Override
	public ResourceProperty<Object> update() throws Exception
	{
		FileSystemDocument fs = xdoc[0];
		fs.getFileSystem().setDescription(DESCRIPTION);
		//fs.getFileSystem().setMountPoint(root);
		
		Long freeSpace = FreeSpaceChecker.get(parentWSResource.getKernel()).getFreeSpace();
		if(freeSpace != null) {
			RangeValueType range = RangeValueType.Factory.newInstance();
			range.addNewExact().setDoubleValue(freeSpace);
			fs.getFileSystem().setDiskSpace(range);
		}

		return this;
	}

	@Override
	public FileSystemDocument[] getXml()
	{
		return xdoc;
	}
}