package de.fzj.unicore.uas.impl.reservation;

import java.util.Calendar;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesDocument;
import org.unigrids.x2006.x04.services.jms.TargetSystemReferenceDocument;
import org.unigrids.x2006.x04.services.reservation.ReservationPropertiesDocument;
import org.unigrids.x2006.x04.services.reservation.ReservationReferenceDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.ReservationManagement;
import de.fzj.unicore.uas.impl.UASWSResourceImpl;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.uas.xnjs.XNJSFacade;
import de.fzj.unicore.wsrflite.messaging.ResourceDeletedMessage;
import de.fzj.unicore.wsrflite.persistence.Persist;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.rp.ImmutableResourceProperty;

/**
 * WS resource representing a resource reservation
 * 
 * @author schuller
 */
public class ReservationManagementImpl extends UASWSResourceImpl implements ReservationManagement {

	private static final Logger logger=LogUtil.getLogger(LogUtil.SERVICES,ReservationManagementImpl.class);
	
	public static String INITPARAM_RESOURCES="resources";
	public static String INITPARAM_STARTTIME="starttime";
	public static String INITPARAM_TSS_REFERENCE="tss_reference";
	public static String INITPARAM_RESERVATION_REFERENCE="reservation_reference";
	
	@Persist
	private String reservationReference;
	
	@Persist
	private String tssID;
	
	@Override
	public QName getResourcePropertyDocumentQName() {
		return ReservationPropertiesDocument.type.getDocumentElementName();
	}
	
	@Override
	public void initialise(String serviceName, Map<String, Object> initParams) throws Exception {
		super.initialise(serviceName, initParams);
	
		reservationReference=(String)initParams.get(INITPARAM_RESERVATION_REFERENCE);
		ReservationReferenceDocument res=ReservationReferenceDocument.Factory.newInstance();
		res.setReservationReference(reservationReference);
		properties.put(RPReservationReference, new ImmutableResourceProperty(res));
		
		//start time
		properties.put(RPStartTime, new StartTimeResourceProperty(reservationReference,(Calendar)initParams.get(INITPARAM_STARTTIME),this,xnjsReference));
		
		//reservation status
		properties.put(RPReservationStatus, new ReservationStatusResourceProperty(reservationReference,this,xnjsReference));
		
		//requested resources
		ResourcesDocument resources=(ResourcesDocument)initParams.get(INITPARAM_RESOURCES);
		properties.put(RPResources, new ImmutableResourceProperty(resources));
		
		TargetSystemReferenceDocument tssRef=TargetSystemReferenceDocument.Factory.newInstance();
		EndpointReferenceType tssEPR=(EndpointReferenceType)initParams.get(INITPARAM_TSS_REFERENCE);
		tssRef.setTargetSystemReference(tssEPR);
		tssID=WSServerUtilities.extractResourceID(tssEPR);
		properties.put(RPTargetSystemReference, new ImmutableResourceProperty(tssRef));
		
	}

	/**
	 * on destroy(), cancel the booking on the backend.
	 * Also, send a message to parent TSS
	 */
	@Override
	public void destroy() {
		try{
			ResourceDeletedMessage m=new ResourceDeletedMessage("deleted:"+getUniqueID());
			m.setServiceName(getServiceName());
			m.setDeletedResource(getUniqueID());
			kernel.getMessaging().getChannel(tssID).publish(m);
		}
		catch(Exception e){
			LogUtil.logException("Could not send internal message.",e,logger);
		}
		try{
			XNJSFacade.get(xnjsReference,kernel).getReservation().cancelReservation(reservationReference, getClient());
		}
		catch(Exception e){
			LogUtil.logException("Could not cancel resource reservation.",e,logger);
		}
		super.destroy();
	}
	
	
	
}
