/*********************************************************************************
 * Copyright (c) 2006-2008 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
 

package de.fzj.unicore.uas.impl.reservation;

import org.unigrids.x2006.x04.services.reservation.ReservationStatusDocument;
import org.unigrids.x2006.x04.services.reservation.ReservationStatusType;
import org.unigrids.x2006.x04.services.reservation.ReservationStatusType.Enum;

import de.fzj.unicore.uas.impl.UASWSResourceImpl;
import de.fzj.unicore.uas.xnjs.XNJSFacade;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.xmlbeans.ResourceProperty;
import de.fzj.unicore.xnjs.ems.ExecutionException;
import de.fzj.unicore.xnjs.tsi.ReservationStatus;
import eu.unicore.security.Client;

/**
 * Represents the status of a resource reservation
 * @since 1.1.1
 */
public class ReservationStatusResourceProperty extends ResourceProperty<Object> {
	private static final long serialVersionUID=1L;
	
	private final ReservationStatusDocument status;
	private final ReservationStatusDocument[] xdoc;
	private final String reservationID;
	private final String xnjsReference;
	
	/**
	 * 
	 * @param jobID the id of the job
	 */
	public ReservationStatusResourceProperty(String reservationID, ReservationManagementImpl parent, String xnjsReference){
		super(parent);
		status=ReservationStatusDocument.Factory.newInstance();
		status.setReservationStatus(ReservationStatusType.UNDEFINED);
		xdoc=new ReservationStatusDocument[]{status};
		this.reservationID=reservationID;
		this.xnjsReference=xnjsReference;
	}

	@Override
	public ReservationStatusDocument[] getXml() {
		return xdoc;
	}

	/**
	 *
	 * @see de.fzj.unicore.wsrflite.ResourceProperty#update()
	 */
	@Override
	public ReservationStatusResourceProperty update() throws Exception{
		try{
			Kernel k=parentWSResource.getKernel();
			Client client=((UASWSResourceImpl)parentWSResource).getClient();
			ReservationStatus reservationStatus=XNJSFacade.get(xnjsReference,k).getReservation().queryReservation(reservationID, client);
			status.setReservationStatus(convertStatus(reservationStatus));
		}catch(ExecutionException ex){
			status.setReservationStatus(ReservationStatusType.UNDEFINED);
		}
		return this;
	}
	
	/**
	 * Converts from the XNJS reservation status type to UNICORE job status
	 * 
	 * @param emsStatus
	 * @return UNICORE status
	 */
	protected Enum convertStatus(ReservationStatus xnjsStatus){
		ReservationStatus.Status  reservationStatus = xnjsStatus.getStatus();
		if(ReservationStatus.Status.UNKNOWN.equals(reservationStatus)){
			return  ReservationStatusType.UNDEFINED;
		}
		//other states map by name
		return ReservationStatusType.Enum.forString(reservationStatus.toString());
	}

	
}
