package de.fzj.unicore.uas.impl.sms;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.FileSystemDocument;

import de.fzj.unicore.uas.xnjs.XNJSFacade;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.xnjs.io.XnjsStorageInfo;
import de.fzj.unicore.xnjs.tsi.TSI;
import eu.unicore.security.Client;

/**
 * provides information about the storage backed by a filesystem
 */
public class DefaultStorageInfoProvider implements StorageInfoProvider {

	private final Kernel kernel;
	
	public DefaultStorageInfoProvider(Kernel kernel){
		this.kernel=kernel;
	}
	
	public FileSystemDocument getInformation(String type, Client client, String xnjsRef) {
		
		String workdir=kernel.getProperty(StorageFactoryImpl.PREFIX+"."+type+".path");
		if("DEFAULT".equals(type) && workdir==null){
			workdir=kernel.getProperty(StorageFactoryImpl.PATH);
		}
		TSI tsi=XNJSFacade.get(xnjsRef,kernel).getTSI(client);
		XnjsStorageInfo info=tsi.getAvailableDiskSpace(workdir);
		FileSystemDocument fs=FileSystemDocument.Factory.newInstance();
		fs.addNewFileSystem().setMountPoint(workdir);
	
		String description=kernel.getProperty(StorageFactoryImpl.PREFIX+"."+type+".description");
		if("DEFAULT".equals(type) && description==null){
			description="Filesystem";
		}
		fs.getFileSystem().setDescription(description);
		
		if(info.getUsableSpace()>-1){
			fs.getFileSystem().addNewDiskSpace().addNewExact().setDoubleValue(info.getUsableSpace());
		}
		else if(info.getFreeSpace()>-1){
			fs.getFileSystem().addNewDiskSpace().addNewExact().setDoubleValue(info.getFreeSpace());
		}
		return fs;	
	}
}
