/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
 

package de.fzj.unicore.uas.impl.tss.rp;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.RangeValueType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.TotalResourceCountDocument;

import de.fzj.unicore.uas.xnjs.XNJSFacade;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.Resource;
import de.fzj.unicore.wsrflite.impl.ResourceImpl;
import eu.unicore.security.Client;

/**
 * represents the number of Nodes on a TargetSystem resource<br>
 */
public class NodesResourceProperty extends IDBBasedResourceProperty<Object> {

	private static final long serialVersionUID = -1641168248769350917L;

	private TotalResourceCountDocument[] xdoc;

	public NodesResourceProperty(Resource parent,String xnjsReference) {
		super(parent,xnjsReference);
	}

	/**
	 * updates Nodes information from NJS
	 */
	@Override
	public NodesResourceProperty update(){
		if(mustUpdate()){
			try{
				Kernel kernel=parentWSResource.getKernel();
				Client client = ((ResourceImpl) parentWSResource).getClient();
				ResourcesType res = XNJSFacade.get(xnjsReference,kernel).getResources(client);
				TotalResourceCountDocument nodesDoc=TotalResourceCountDocument.Factory.newInstance();
				RangeValueType nodes = res.getTotalResourceCount();
				if(nodes != null){
					nodesDoc.setTotalResourceCount(nodes);
					xdoc = new TotalResourceCountDocument[1];
					xdoc[0]=nodesDoc;
				}
				else{
					logger.debug("Number of nodes (TotalResourceCount) is not set in IDB.");
					xdoc = new TotalResourceCountDocument[0];
				}
			}catch(Exception e){
				logger.warn(
						"Problem updating Nodes resourceproperty.",e);
				xdoc = new TotalResourceCountDocument[0];
			}
		}
		return this;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see de.fzj.unicore.wsrflite.ResourceProperty#getXml()
	 */
	public TotalResourceCountDocument[] getXml() {
		return xdoc;
	}





}
