/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
 


package de.fzj.unicore.uas.util;

import org.apache.log4j.Logger;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.impl.registry.MulticastRegistryAdvertiser;
import de.fzj.unicore.uas.impl.registry.RegistryHandler;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.KernelInjectable;

/**
 * Starts advertising the Registry using UDP multicast
 * To run on uas startup, set a property "uas.onstartup" to
 * contain the string de.fzj.unicore.uas.util.RegistryStartup
 */
public class RegistryStartup implements Runnable, KernelInjectable {

	private static final Logger logger=LogUtil.getLogger(LogUtil.UNICORE,RegistryStartup.class);

	private Kernel kernel;
	
	public RegistryStartup(){
	}
	

	public RegistryStartup(Kernel kernel){
		this.kernel=kernel;
	}
	
	@Override
	public void setKernel(Kernel kernel) {
		this.kernel=kernel;
	}

	public void run(){
		
		try
		{
			logger.info("Setting up shared registry");
			RegistryHandler.get(kernel);
			if(Boolean.parseBoolean(kernel.getProperty(UAS.GLOBAL_REGISTRY_ADVERTISE, "false"))){
				logger.info("Advertising global registry via multicast!");
				MulticastRegistryAdvertiser rma = new MulticastRegistryAdvertiser(kernel);
				rma.run();
			}
			
		} catch (Exception e) {
			logger.fatal("Error creating shared registry",e);
		}
		
	}
}

