/*********************************************************************************
 *  Copyright (c) 2006-2010 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/

package de.fzj.unicore.uas.xnjs;

import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.xnjs.Configuration;
import de.fzj.unicore.xnjs.aaa.ISecurityConfiguration;

/**
 * This class provides a security configuration for the XNJS, which is used
 * e.g. for making SSL connections to the Perl TSI. To activate SSL for the 
 * XNJS to TSI communication, it is necessary to load this class in the XNJS 
 * config file.</br>
 * In the xnjs config file, add a line to the "Core" section: </br>
 * <code>
 *   &lt;x:LoadComponent xmlns:x="http://www.fz-juelich.de/unicore/xnjs/engine"&gt;de.fzj.unicore.uas.xnjs.XNJSSecurityConfiguration&lt;/x:LoadComponent&gt;
 * </code>
 * @author schuller
 */
public class XNJSSecurityConfiguration implements ISecurityConfiguration {

	private final Kernel kernel;
	
	public XNJSSecurityConfiguration(Configuration config){
		this.kernel=config.getComponentInstanceOfType(Kernel.class);
	}
	
	public String getKeystore() {
		return kernel.getSecurityProperties().getKeystore();
	}

	public String getKeystorePassword() {
		return kernel.getSecurityProperties().getKeystorePassword();
	}

	public String getKeystoreType() {
		return kernel.getSecurityProperties().getKeystoreType();
	}

	public String getTruststore() {
		return kernel.getSecurityProperties().getTruststore();
	}

	public String getTruststorePassword() {
		return kernel.getSecurityProperties().getTruststorePassword();
	}

	public String getTruststoreType() {
		return kernel.getSecurityProperties().getTruststoreType();
	}

	public boolean isSSL() {
		return kernel.getSecurityProperties().isSslEnabled();
	}

	public boolean isSSLClientAuth() {
		return kernel.getSecurityProperties().requireClientAuthentication();
	}

}
