package de.fzj.unicore.uas.dsms;

import java.util.HashMap;
import java.util.Map;

import junit.framework.Assert;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.impl.sms.StorageManagementHomeImpl;
import de.fzj.unicore.uas.impl.sms.StorageManagementHomeImpl.StorageTypes;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.exceptions.ResourceNotCreatedException;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.xmlbeans.impl.WSResourceImpl;
import eu.unicore.security.util.client.HttpUtils;

public abstract class DsmsTestBase extends AbstractTestBase
{
	private Home dSmsHome;
	private String dSmsResId;
	
	protected void createDsms() throws ResourceNotCreatedException
	{
		String defaultDSmsName = "default_dsms";
		Map<String,Object> map = new HashMap<String,Object>();
		map.put(WSResourceImpl.INIT_UNIQUE_ID, defaultDSmsName);
		map.put(WSResourceImpl.INIT_INITIAL_TERMINATION_TIME, lifeTime());
		map.put(StorageManagementHomeImpl.INIT_STORAGETYPE, StorageTypes.DSMS);
		
		dSmsHome = kernel.getHome(UAS.SMS);
		Assert.assertNotNull(dSmsHome);
		dSmsResId = dSmsHome.createWSRFServiceInstance(map);
	}
	
	protected void destroyDsms() throws Exception
	{
		dSmsHome.destroyWSRFServiceInstance(dSmsResId);
	}
	
	protected StorageClient getDsmsClient() throws Exception
	{
		String url = kernel.getProperty(Kernel.WSRF_BASEURL);
		EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
		epr.addNewAddress().setStringValue(url + "/" + UAS.SMS + "?res=default_dsms");
		ISecurityProperties clientProp = kernel.getSecurityProperties();
		clientProp.setProperty(HttpUtils.CONNECT_TIMEOUT, "6000000");
		clientProp.setProperty(HttpUtils.SO_TIMEOUT, "6000000");
		return new StorageClient(epr, clientProp);
	}
}