package de.fzj.unicore.uas.dsms.stress;

import java.util.Random;

import de.fzj.unicore.uas.client.RByteIOClient;
import de.fzj.unicore.wsrflite.xmlbeans.BaseFault;

enum Operation {
	IMPORT(3) {
		@Override
		public void perform(StressTestPerformer performer) throws Exception {
			String fileName = performer.getNewFile();
			RByteIOClient transfer = performer.getClient().getImport(fileName);
			transfer.write(BODY);
			transfer.destroy();
		}
	},
	EXPORT(3) {
		@Override
		public void perform(StressTestPerformer performer) throws Exception {
			String fileName = performer.getRandomFile();
			if(fileName == null) {
				return;
			}
			RByteIOClient transfer = performer.getClient().getExport(fileName);
			transfer.read(0, 1, BODY.length, 0);
		}
	},
	/*MKDIR(5) {
		@Override
		public void perform(StressTestPerformer performer) throws BaseFault {
			String dirName = performer.getNewDir();
			performer.getClient().createDirectory(dirName);
		}
	},*/
	RENAME(1) {
		@Override
		public void perform(StressTestPerformer performer) throws BaseFault {
			String oldFile = performer.deleteRandomFile();
			if(oldFile == null) {
				return;
			}
			String newFile = performer.getNewFile();
			performer.getClient().rename(oldFile, newFile);
		}
	},
	COPY(1) {
		@Override
		public void perform(StressTestPerformer performer) throws BaseFault {
			String oldFile = performer.getRandomFile();
			if(oldFile == null) {
				return;
			}
			String newFile = performer.getNewFile();
			performer.getClient().copy(oldFile, newFile);
		}
	},
	DELETE(1) {
		@Override
		public void perform(StressTestPerformer performer) throws BaseFault {
			String fileName = performer.deleteRandomFile();
			if(fileName == null) {
				return;
			}
			performer.getClient().delete(fileName);
		}
	};

	private static int probSum;

	private static final Random RANDOM = new Random();

	private static final byte[] BODY = "sample file body".getBytes();

	static {
		for (Operation o : Operation.values()) {
			probSum += o.getProbability();
		}
	}

	private int probability;

	private Operation(int probability) {
		this.probability = probability;
	}

	private int getProbability() {
		return probability;
	}

	public abstract void perform(StressTestPerformer performer)
			throws Exception;

	public static Operation getRandomOperation() {
		int r = RANDOM.nextInt(probSum) + 1;
		int s = 0;

		Operation op = null;
		for (Operation o : Operation.values()) {
			s += o.getProbability();
			if (s >= r) {
				return o;
			}
			op = o;
		}
		return op;
	}
}