package de.fzj.unicore.uas.dsms.stress;

import static org.junit.Assert.fail;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import pl.edu.icm.dsms.catalogue.db.InitDB;
import de.fzj.unicore.uas.Base;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.dsms.AbstractTestBase;
import de.fzj.unicore.uas.dsms.DsmsTestBase;
import de.fzj.unicore.uas.util.LogUtil;
import eu.unicore.security.util.Log;

@Ignore
public class StressTest extends DsmsTestBase {
	private static final Logger logger = Log.getLogger(LogUtil.CLIENT,
			StressTest.class);
	
	private static final int THREADS_NO = 2;
	
	private static final int ITERATIONS = 50;
	
	private String dataMap="src/test/resources/dcatalogue/datamap.properties";
	
	@BeforeClass
	public static void startKernel()
	{
		try
		{
			Base.initDirectories();
			System.out.println("Starting kernel");
			AbstractTestBase.startKernel();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Failed to start kernel");
		}
	}

	@AfterClass
	public static void shutdownKernel()
	{
		try
		{
			System.out.println("Shutting down kernel");
			AbstractTestBase.shutdownKernel();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Failed to shutdown kernel");
		}
	}

	@Before
	public void setUp() throws Exception
	{
		cleanStorageDirs();
		InitDB.initDB(dataMap);
		createSms();
		createCatalogue();
		createDsms();
	}

	@After
	public void tearDown()
	{
		try
		{
			destroyDsms();
			destroyCatalogue();
			destroySms();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Unable to tear down");
		}
	}

	@Test
	public void stressTest() throws Exception {
		Thread[] threads = new Thread[THREADS_NO];
		
		logger.info("Creating " + THREADS_NO + " threads");
		for(int i=0;i<THREADS_NO;i++) {
			StorageClient client = getDsmsClient();
			String parentDir = "thread_" + i;
			client.createDirectory(parentDir);
			StressTestPerformer performer = new StressTestPerformer(parentDir, client, ITERATIONS);
			threads[i] = new Thread(performer);
		}

		for(int i=0;i<THREADS_NO;i++) {
			logger.info("Starting thread " + i);
			threads[i].start();
		}
		
		for(int i=0;i<THREADS_NO;i++) {
			logger.info("Waiting for thread " + i);
			threads[i].join();
		}
	}
}
