package de.fzj.unicore.uas.dsms.stress;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.log4j.Logger;

import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.util.LogUtil;
import eu.unicore.security.util.Log;

public class StressTestPerformer implements Runnable {
	private static final Logger logger = Log.getLogger(LogUtil.CLIENT,
			StressTestPerformer.class);

	private final List<String> fileNames;

	private final DirectoryList dirList;

	private final Random random;

	private final StorageClient client;

	private final int iterations;
	
	private int fileId;
	
	private String parentDir;

	public StressTestPerformer(String parentDir, StorageClient client,
			int iterations) {
		this.fileNames = new ArrayList<String>();
		this.dirList = new DirectoryList(50, 5, parentDir);
		this.random = new Random();
		this.client = client;
		this.iterations = iterations;
		this.parentDir = parentDir;
	}

	@Override
	public void run() {
		try {
			for (int i = 0; i < 10; i++) {
				Operation.IMPORT.perform(this);
			}
			for (int i = 0; i < iterations; i++) {
				Operation.getRandomOperation().perform(this);
			}
			client.delete(parentDir);
		} catch (Exception e) {
			logger.error("Can't perform operation", e);
		}
	}

	String getNewFile() {
		String dirName = dirList.getRandom().getDirName();
		String fileName = dirName + "/file_" + (fileId++);
		fileNames.add(fileName);
		return fileName;
	}

	String getRandomFile() {
		int index = randomFileIndex();
		if(index == -1) {
			return null;
		}
		return fileNames.get(index);
	}
	
	String deleteRandomFile() {
		int index = randomFileIndex();
		if(index == -1) {
			return null;
		}
		return fileNames.remove(index);
	}

	String getRandomDir() {
		return dirList.getRandom().getDirName();
	}

	StorageClient getClient() {
		return client;
	}

	private int randomFileIndex() {
		if(fileNames.size() == 0) {
			return -1;
		}
		return random.nextInt(fileNames.size());
	}
}
