/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
 
package de.fzj.unicore.uas.impl;

import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

import org.apache.xmlbeans.XmlDateTime;
import org.oasisOpen.docs.wsrf.rp2.GetResourcePropertyDocument;
import org.oasisOpen.docs.wsrf.sg2.AddDocument;
import org.oasisOpen.docs.wsrf.sg2.AddResponseDocument;
import org.oasisOpen.docs.wsrf.sg2.EntryType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.Registry;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.LocalRegistryClient;
import de.fzj.unicore.uas.impl.registry.LocalRegistryEntryHomeImpl;
import de.fzj.unicore.uas.impl.registry.LocalRegistryHomeImpl;
import de.fzj.unicore.uas.impl.registry.RegistryEntryImpl;
import de.fzj.unicore.uas.impl.registry.RegistryHandler;
import de.fzj.unicore.uas.impl.registry.RegistryImpl;
import de.fzj.unicore.wsrflite.FaultConverter;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Resource;
import de.fzj.unicore.wsrflite.exceptions.ResourceUnknownException;
import de.fzj.unicore.wsrflite.impl.DefaultHome;
import de.fzj.unicore.wsrflite.xfire.LocalTestCase;
import de.fzj.unicore.wsrflite.xfire.XFireServiceFactory;
import de.fzj.unicore.wsrflite.xmlbeans.sg.impl.ServiceGroupImpl;
import de.fzj.unicore.wsrflite.xmlbeans.sg.impl.ServiceGroupRegistrationImpl;

public class TestRegistry extends LocalTestCase {

	@Override
	protected void addServices() throws Exception {
		XFireServiceFactory.createAndDeployService(kernel,UAS.REG,	Registry.class,LocalRegistryHomeImpl.class,false);
		XFireServiceFactory.createAndDeployService(kernel,UAS.REGENTRY,Registry.class,LocalRegistryEntryHomeImpl.class,false);
	}
	
	public void testSetLifetime() throws Exception{
		ServiceGroupRegistrationImpl sg=new ServiceGroupRegistrationImpl();
		sg.setKernel(kernel);
		sg.setHome(new DefaultHome(){
			public FaultConverter getFaultConverter(){
				return new FaultConverter(){
					public Exception convert(Exception e) {
						return e;
					}
				};
			}
			protected Resource doCreateInstance() {return null;}
			
			@Override
			public String extractServiceReference(Object context)
					throws ResourceUnknownException {
				return null;
			}
			
		});
		HashMap<String,Object>map=new HashMap<String,Object>();
		Calendar c=Calendar.getInstance();
		c.add(Calendar.YEAR,50);
		sg.initialise("Registry",map);
		EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
		epr.addNewAddress().setStringValue("http://fooooo");
		AddDocument in=AddDocument.Factory.newInstance();
		in.addNewAdd().setMemberEPR(epr);
		in.getAdd().addNewContent();
		XmlDateTime tt=XmlDateTime.Factory.newInstance();
		tt.setCalendarValue(c);
		in.getAdd().setInitialTerminationTime(tt);
		AddResponseDocument res=sg.Add(in);
		Calendar responseTT=res.getAddResponse().getTerminationTime();
		assertTrue(responseTT.getTimeInMillis()==c.getTimeInMillis());
	}
	
	public void testDestroy() throws Exception{
		kernel.setProperty("expirycheck.initial","1"); 
		kernel.setProperty("expirycheck.period","1"); 
		Home sgeHome=kernel.getHome(UAS.REGENTRY);
		sgeHome.activateHome(UAS.REGENTRY);
		Home sgHome=kernel.getHome(UAS.REG);
		sgHome.activateHome(UAS.REG);
		
		RegistryImpl sg=new RegistryImpl();
		sg.setHome(sgHome);
		sg.setKernel(kernel);
		HashMap<String,Object>map=new HashMap<String,Object>();
		sg.initialise(UAS.REG,map);
		EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
		epr.addNewAddress().setStringValue("http://fooooo");
		AddDocument in=AddDocument.Factory.newInstance();
		in.addNewAdd().setMemberEPR(epr);
		in.getAdd().addNewContent();
		AddResponseDocument res=sg.Add(in);
		
		EndpointReferenceType sge=res.getAddResponse().getServiceGroupEntryReference();
		assertNotNull(sge);
		String resid=sge.getAddress().getStringValue().split("=")[1];
		RegistryEntryImpl sgeInstance=null;
		try{
			sgeInstance=(RegistryEntryImpl)sgeHome.get(resid);
			assertNotNull(sgeInstance);
			GetResourcePropertyDocument req=GetResourcePropertyDocument.Factory.newInstance();
			req.setGetResourceProperty(ServiceGroupImpl.RPEntry);
			assertTrue(sg.GetResourceProperty(req).toString().contains(resid));
			sgeInstance.destroy();
			sg.postActivate();
			assertFalse(sg.GetResourceProperty(req).toString().contains(resid));
		}
		catch(Exception e){
			System.out.println("Got: "+e.getMessage());
		}
	}
	
	
	public void testListEntries()throws Exception{
		LocalRegistryClient lrc=RegistryHandler.get(kernel).getRegistryClient();
		EndpointReferenceType memberEpr=EndpointReferenceType.Factory.newInstance();
		memberEpr.addNewAddress().setStringValue("https://foo");
		int before=lrc.listEntries().size();
		lrc.addRegistryEntry(memberEpr);
		int diff=lrc.listEntries().size()-before;
		List<EntryType>entries=lrc.listEntries();
		assertNotNull(entries);
		assertTrue(entries.size()>0);
		assertEquals(1,diff);
	}
	
}
