package de.fzj.unicore.uas.security;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.codehaus.xfire.service.Service;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.oasisOpen.docs.wsrf.rp2.GetResourcePropertyDocument;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.security.testservices.ExampleService;
import de.fzj.unicore.uas.security.testservices.IExample;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.security.UASSecurityProperties;
import de.fzj.unicore.wsrflite.xfire.WSRFClientFactory;
import de.fzj.unicore.wsrflite.xfire.XFireKernel;
import de.fzj.unicore.wsrflite.xfire.XFireServiceFactory;
import eu.unicore.security.SecurityTokens;

public class TestProxyHandlers extends TestCase {

	Kernel kernel;
	
	@Override
	protected void setUp()throws Exception{
		UAS uas=new UAS("src/test/resources/secure/uas.config.simple");
		uas.startSynchronous();
		kernel=uas.getKernel();
		addServices();
	}


	protected void addServices()throws Exception{
		String serviceName="testProxyHandlers";
		XFireServiceFactory.createAndDeployService(kernel,serviceName,IExample.class, ExampleService.class, false);
		Service s=XFireKernel.get(kernel).getService(serviceName);
		s.addInHandler(new ProxyCertInHandler(kernel));
	}

	private static String secPropsOut=
		ISecurityProperties.WSRF_SSL + "=true"+"\n"+
		ISecurityProperties.WSRF_SSL_CLIENTAUTH + "=true" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYPASS + "=the!user"+"\n"+
		ISecurityProperties.WSRF_SSL_KEYTYPE + "=jks"+"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTPASS + "=the!user\n";

	public void test()throws Exception{
		doTestOutHandler();
		doTestInHandler();
	}

	private void doTestOutHandler()throws Exception{
		// establish security
		ISecurityProperties sec=new UASSecurityProperties(kernel.getProperties(), 
				new ByteArrayInputStream(secPropsOut.getBytes()));
		ProxyCertOutHandler p=new ProxyCertOutHandler();
		p.configure(sec);
		Element ph=p.getProxyHeader();
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		new XMLOutputter(Format.getPrettyFormat()).output(ph,bos);
		assertTrue(bos.toString().contains("BEGIN RSA PRIVATE KEY"));
	}

	
	private void doTestInHandler()throws Exception{
		ISecurityProperties sec=new UASSecurityProperties(kernel.getProperties(),
				new ByteArrayInputStream(secPropsOut.getBytes()));
		sec.setProperty(ISecurityProperties.UAS_OUTHANDLER_NAME, ProxyCertOutHandler.class.getName());
		
		IExample c=new WSRFClientFactory(sec).createPlainWSProxy(IExample.class, getBaseurl()+"/testProxyHandlers");
		GetResourcePropertyDocument in=GetResourcePropertyDocument.Factory.newInstance();
		in.setGetResourceProperty(new QName("Hello","World"));
		c.getTime(in);
		
		SecurityTokens st=ExampleService.getLastCallSecurityTokens();
		assertNotNull(st);
		Object proxy=st.getContext().get(ProxyCertInHandler.PROXY);
		assertNotNull(proxy);
		assertTrue(proxy.toString().contains("PRIVATE KEY"));
	}

	private String getBaseurl(){
		return kernel.getProperty(Kernel.WSRF_BASEURL);
	}
	
}
