package de.fzj.unicore.uas.testsuite;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.List;

import org.junit.Test;
import org.oasisOpen.docs.wsrf.sg2.EntryType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.Base;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.impl.registry.RegistryEntryUpdater;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.BaseWSRFClient;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import eu.unicore.bugsreporter.annotation.FunctionalTest;

/**
 * tests for the registry "list services" function
 */
public class TestRegistryEntryUpdater extends Base {

	String url;
	EndpointReferenceType registryEpr;

	@FunctionalTest(id="TestEntryUpdater", description="Tests the registry entry updater mechanism")
	@Test
	public void testEntryUpdater()throws Exception{
		url=kernel.getProperty(Kernel.WSRF_BASEURL);
		registryEpr=EndpointReferenceType.Factory.newInstance();
		registryEpr.addNewAddress().setStringValue(url+"/"+UAS.REG+"?res=default_registry");
		RegistryClient reg=new RegistryClient(registryEpr,kernel.getSecurityProperties());
		//list entries and figure out storage entry EPR and registry entry EPR
		List<EntryType>entries=reg.listEntries();
		assertNotNull(entries);
		assertTrue(entries.size()>0);
		EndpointReferenceType storageEPR=null;
		EndpointReferenceType entryEPR=null;
		String entryID=null;
		for(EntryType e: entries){
			EndpointReferenceType member=e.getMemberServiceEPR();
			String memberURL=member.getAddress().getStringValue();
			if(memberURL.endsWith("default_storage")){
				storageEPR=member;
				entryEPR=e.getServiceGroupEntryEPR();
				entryID=WSUtilities.extractResourceID(entryEPR);
				break;
			}
		}
		assertNotNull(storageEPR);
		assertNotNull(entryEPR);
		assertNotNull(entryID);
		StorageClient sms=new StorageClient(storageEPR, kernel.getSecurityProperties());
		BaseWSRFClient entry=new BaseWSRFClient(entryEPR, kernel.getSecurityProperties());
		Home registryHome=kernel.getHome(UAS.REGENTRY);
		assertNotNull(registryHome);
		RegistryEntryUpdater upd=new RegistryEntryUpdater();
		boolean expired=upd.check(registryHome, entryID);
		assertFalse(expired);
	
		//check if process() returns true (since the service is up)
		boolean up=upd.process(registryHome, entryID);
		assertTrue(up);
		
		//remove the service
		sms.destroy();
		
		//and process() should be false now
		up=upd.process(registryHome, entryID);
		assertFalse(up);
		
		//and the registry entry should be gone
		try{
			entry.getCurrentTime();
			fail("Entry should be gone");
		}catch(Exception ex){
			/* OK */
		}
	
	}
	
	@Test
	public void testChangeBaseURL()throws Exception{
		url=kernel.getBaseURL();
		registryEpr=EndpointReferenceType.Factory.newInstance();
		registryEpr.addNewAddress().setStringValue(url+"/"+UAS.REG+"?res=default_registry");
		RegistryClient reg=new RegistryClient(registryEpr,kernel.getSecurityProperties());
		//add a fake plain ws entry
		EndpointReferenceType memberEpr=EndpointReferenceType.Factory.newInstance();
		memberEpr.addNewAddress().setStringValue(url+"Test");
		reg.addRegistryEntry(memberEpr);
		
		//list entries and figure out correct registry entry EPR
		List<EntryType>entries=reg.listEntries();
		assertNotNull(entries);
		assertTrue(entries.size()>0);
		EndpointReferenceType entryEPR=null;
		String entryID=null;
		for(EntryType e: entries){
			EndpointReferenceType member=e.getMemberServiceEPR();
			String memberURL=member.getAddress().getStringValue();
			if(memberURL.contains("Test")){
				entryEPR=e.getServiceGroupEntryEPR();
				entryID=WSUtilities.extractResourceID(entryEPR);
				break;
			}
		}
		assertNotNull(entryEPR);
		assertNotNull(entryID);
		
		//change base url
		kernel.setProperty(Kernel.WSRF_BASEURL, "x"+url);

		Home registryHome=kernel.getHome(UAS.REGENTRY);
		assertNotNull(registryHome);

		RegistryEntryUpdater upd=new RegistryEntryUpdater();
		boolean valid=upd.process(registryHome, entryID);
		assertFalse(valid);
		
		//and the registry entry should be gone
		try{
			registryHome.get(entryID);
			fail("Entry should be gone");
		}catch(Exception ex){
			/* OK */
		}
	
	}
	

}
