package de.fzj.unicore.uas.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.junit.Test;

public class TestTimeoutRunner {

	private final ExecutorService exec=Executors.newSingleThreadExecutor();

	Callable<String>task1=new Callable<String>(){
		public String call() throws Exception{
			for(int t=0;t<10;t++){
				Thread.sleep(100);
			}
			return "Result";
		}
	};
	
	Callable<String>task2=new Callable<String>(){
		public String call() throws Exception{
			for(int t=0;t<3;t++){
				Thread.sleep(100);
			}
			return "Result";
		}
	};
	
	@Test
	public void testTaskWithTimeout(){
		//call with short timeout
		String result=TimeoutRunner.compute(task1, 100, Executors.newSingleThreadExecutor());
		assertNull(result);
	}
	
	@Test
	public void testTaskWithoutTimeout(){
		//call with long timeout
		String result=TimeoutRunner.compute(task1, 2000, exec);
		assertNotNull(result);
		assertEquals("Result",result);
	}

	@Test
	public void testAccuracyOfTimeout()throws InterruptedException{
		//call with short timeout
		String result=TimeoutRunner.compute(task2, 180, exec);
		assertNull(result);
		Thread.sleep(1000);
		//call with long enough timeout
		result=TimeoutRunner.compute(task2, 320, exec);
		assertNotNull(result);
	}

	@Test
	public void testTaskWithTimeoutUsingUnits(){
		//call with short timeout
		String result=TimeoutRunner.compute(task1, 100000, TimeUnit.MICROSECONDS, exec);
		assertNull(result);
	}
	
	
	@Test
	public void testTaskWithoutTimeoutUsingUnits(){
		//call with long timeout
		String result=TimeoutRunner.compute(task1, 2,TimeUnit.SECONDS, exec);
		assertNotNull(result);
		assertEquals("Result",result);
	}

}
