package de.fzj.unicore.uas.functional;

import java.io.ByteArrayOutputStream;
import java.util.Calendar;
import java.util.List;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.unigrids.x2006.x04.services.tss.SubmitDocument;
import org.unigrids.x2006.x04.services.tss.SubmitResponseDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.BaseUASClient;
import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.RByteIOClient;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.wsrflite.Kernel;

public abstract class AbstractJobRun extends RegressionTest {

	protected String url;
	
	protected EndpointReferenceType tssepr;
	
	protected TSSClient tss;
	
	protected JobClient job;
	
	protected Kernel kernel;
	
	@Override
	public void doTest(Kernel k) {
		try{
			this.kernel=k;
			url=kernel.getProperty(Kernel.WSRF_BASEURL);
			tssepr=EndpointReferenceType.Factory.newInstance();
			tssepr.addNewAddress().setStringValue(url+"/"+UAS.TSS+"?res=default_targetsystem");
			tss=new TSSClient(tssepr, kernel.getSecurityProperties());
			Calendar c=tss.getCurrentTime();
			assertTrue(c!=null);
			runJob(tss);
		}catch(Exception e){
			onError();
			throw new RuntimeException(e);
		}
	}

	protected EndpointReferenceType findFirstAccessibleService(List<EndpointReferenceType>eprs){
		for(EndpointReferenceType epr: eprs){
			try{
				BaseUASClient c=new BaseUASClient(epr, kernel.getSecurityProperties());
				c.getCurrentTime();
				return epr;
			}catch(Exception e){}
		}
		return null;
	}
	
	protected void onError(){
		
		
	}
	
	protected void runJob(TSSClient tss) throws Exception{
		job=submitJob(tss);
		beforeStart(job);
		job.waitUntilReady(180*1000);
		job.start();
		job.waitUntilDone(180*1000);
		onFinish(job);
	}
	/**
	 * actions to be done by the client before start of the job
	 */
	protected abstract void beforeStart(JobClient jms) throws Exception;
	
	
	/**
	 * actions to be done by the client after completion the job
	 */
	protected abstract void onFinish(JobClient jms) throws Exception;
	
	/**
	 * performs job submission
	 * @param tss
	 * @return
	 * @throws Exception
	 */
	protected JobClient submitJob(TSSClient tss)throws Exception{
		JobDefinitionDocument jdd=getJob();
		SubmitDocument req=SubmitDocument.Factory.newInstance();
		req.addNewSubmit().setJobDefinition(jdd.getJobDefinition());
		SubmitResponseDocument res=tss.Submit(req);
		return new JobClient(res.getSubmitResponse().getJobReference(), kernel.getSecurityProperties());
	}
	
	protected String getStdout(JobClient jms)throws Exception{
		return new String(stageOut(jms,"/stdout"));
	}
	
	protected String getStderr(JobClient jms)throws Exception{
		return new String(stageOut(jms,"/stderr"));
	}
	
	protected void stageIn(JobClient jms,String fileName, byte[] data)throws Exception{
		StorageClient uspace=jms.getUspaceClient();
		RByteIOClient fileClient=uspace.getImport(fileName);
		fileClient.write(data);
	}
	
	protected byte[] stageOut(JobClient jms,String fileName)throws Exception{
		StorageClient uspace=jms.getUspaceClient();
		RByteIOClient fileClient=uspace.getExport(fileName);
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		fileClient.readAllData(bos);
		return bos.toByteArray();
	}
	
	/**
	 * build the job definition for submission
	 * @return
	 */
	protected abstract JobDefinitionDocument getJob();
}
