package de.fzj.unicore.uas.functional;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.unigrids.x2006.x04.services.fts.SummaryType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.TransferControllerClient;

public class RunServerServerTransfer extends AbstractJobRun {

	String url;
	EndpointReferenceType tssepr;

	public RunServerServerTransfer(){
	}

	@Override
	public String getDescription() {
		return "Tests the server-server transfer.";
	}

	@Override
	protected void beforeStart(JobClient jms) throws Exception {
		
		String protocol="UDT";
		System.out.println("\n\nTesting server-server transfer using <"+protocol+">\n\n");
		
		runStorageToUspace(protocol, jms);
		
		System.out.println();
		System.out.println();
		
		runUspaceToStorage(protocol, jms);
	
		
	}

	protected void runStorageToUspace(String protocol, JobClient jms)throws Exception{
		//tell uspace sms to fetch a file from the default_storage sms
		TransferControllerClient c=jms.getUspaceClient().fetchFile(protocol+":http://localhost:65321/services/StorageManagement?res=default_storage#/test.txt",
		"/testfile");
		assertTrue(c!=null);
		jms.waitUntilReady(180*1000);
		int count=0;
		while(!c.isComplete() && count<10){
			Thread.sleep(500);
			if(c.getStatusSummary().equals(SummaryType.FAILED)){
				throw new Exception(c.getStatus());	
			}
			count++;
		}
		System.out.println(c.getResourcePropertyDocument());
		System.out.println("Size of remote file = "+c.getSize());
		System.out.println("Transferred         = "+c.getTransferredBytes());
	}
	
	protected void runUspaceToStorage(String protocol, JobClient jms)throws Exception{
		//tell uspace sms to send a file to the default_storage sms
		//write file first
		jms.getUspaceClient().getImport("/testfile1").append("this is a test file exported from uspace to the default storage".getBytes());

		TransferControllerClient c1=jms.getUspaceClient().sendFile("/testfile",
		protocol+":http://localhost:65321/services/StorageManagement?res=default_storage#test1.txt");
		assertTrue(c1!=null);
		jms.waitUntilReady(180*1000);
		int count=0;
		do{
			//System.out.println("Not complete...");
			Thread.sleep(500);
			count++;
			if(c1.getStatusSummary().equals(SummaryType.FAILED)){
				throw new Exception(c1.getStatus());	
			}
		}while(!c1.isComplete() && count < 5);
		c1.setUpdateInterval(-1);
		System.out.println(c1.getResourcePropertyDocument());
		System.out.println("Size of remote file = "+c1.getSize());
		System.out.println("Transferred         = "+c1.getTransferredBytes());
	}

	
	@Override
	protected void onError(){

	}


	@Override
	protected void onFinish(JobClient jms) throws Exception {

	}

	@Override
	protected JobDefinitionDocument getJob() {
		JobDefinitionDocument jdd=JobDefinitionDocument.Factory.newInstance();
		ApplicationType app=jdd.addNewJobDefinition().addNewJobDescription().addNewApplication();
		app.setApplicationName("Date");
		app.setApplicationVersion("1.0");
		return jdd;
	}
}
