package de.fzj.unicore.uas.functional;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.fts.udt.UDTSession;
import de.fzj.unicore.uas.impl.UASWSResourceImpl;
import de.fzj.unicore.uas.util.CreateSMSOnStartup;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.utils.StopWatch;

/**
 * regression test suite for the UDT data transfer
 * starts a server and runs some functional tests against it
 */
public class TestSuite extends TestCase{

	protected static StopWatch sw=new StopWatch();
	
	private static int errors=0;
	
	protected static ArrayList<RegressionTest> tests=new ArrayList<RegressionTest>();
	
	protected static Kernel kernel;
	
	protected String getConfigPath(){
		return "src/test/resources/uas.config";
	}
	protected void setUp() throws Exception{
		long start=System.currentTimeMillis();
		System.out.println("Starting testsuite, this may take a while...");
		UAS uas=new UAS(getConfigPath());
		uas.startSynchronous();
		kernel=uas.getKernel();
		
		//make default_SMS workdir absolute 
		String smsPath=kernel.getProperty(CreateSMSOnStartup.CONF_WORKDIR);
		if(smsPath!=null){
			kernel.setProperty(CreateSMSOnStartup.CONF_WORKDIR, new File(smsPath).getAbsolutePath());
		}
		//configure locations of send/receive
		File send=new File("src/test/resources/fufex-send");
		File recv=new File("src/test/resources/fufex-recv");
		kernel.setProperty(UDTSession.UDT_SEND, send.getAbsolutePath());
		kernel.setProperty(UDTSession.UDT_RECEIVE, recv.getAbsolutePath());
		
		addTests();
		
		System.err.println("Startup time: "+(System.currentTimeMillis()-start)+" ms.");
		//create a "default" TSS
		Home h=kernel.getHome(UAS.TSS);
		Map<String,Object>initObj=new HashMap<String, Object>();
		initObj.put(UASWSResourceImpl.INIT_UNIQUE_ID,"default_targetsystem");
		h.createWSRFServiceInstance(initObj);
	}
	
	protected void addTests(){
		tests.add(new RunServerServerTransfer());
	//	tests.add(new RunClientServerTransfer());
	}
	
	public void testAll()throws Exception{
		
		sw.snapShot("Init done.");
		for(RegressionTest r: tests){
			sw.snapShot("Start: \n"+r.getDescription());
			try{
				System.out.println("Start test ["+r.getDescription()+"]");
				r.doTest(kernel);
				
				if(r.success()) sw.snapShot("OK.");
				else
				{
					errors++;
					sw.snapShot("Error (Unknown cause)");
				}

			}catch(Exception e){
				e.printStackTrace();
				sw.snapShot("Error: "+e.getMessage());	
				errors++;
			}
		}
		System.out.println("***\n*** Results:\n***\n");
		System.out.println(sw.toString());
		System.out.println(errors+" errors occurred.");
		if(errors>0)fail();
	}

	
	
	
}
